import React, { useState, useCallback, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { csv, scaleLinear, scaleOrdinal, max, format, extent } from 'd3';
import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';
import { ColorLegend } from './ColorLegend';
import ReactDropdown from 'react-dropdown';
import { Dropdown } from './Dropdown';

//canvas size
const width = 960;
const menuHeight = 80;
const height = 500 - menuHeight;
const margin = { top: 20, right: 200, bottom: 65, left: 90 };
const xAxisLabelOffset = 50;
const yAxisLabelOffset = 55;
const fadeOpacity = 0.15;

//dropdown values here
const attributes = [
  { value: 'Retentions', label: 'Retentions' },
  { value: 'Retentions Rate', label: 'Retentions Rate' },
  { value: 'Attendance Rate', label: 'Attendance Rate' },
  { value: 'Average # of Absences', label: 'Average # of Absences' },
  { value: 'Absent 10 or more days', label: 'Absent 10 or more days' },
  { value: 'Unexcused > 9 days', label: 'Unexcused > 9 days' },
];

//getting values
const getLabel = value => {
  for (let i = 0; i < attributes.length; i++) {
    if (attributes[i].value === value) {
      return attributes[i].label;
    }
  }
};


const App = () => {
  const data = useData();
  const [hoveredValue, setHoveredValue] = useState(null);

//X Value  
  const initialXAttribute = 'Chronically_Absent';
  const [xAttribute, setXAttribute] = useState(initialXAttribute);
  const xValue = d => d[xAttribute];
  const xAxisLabel = getLabel(xAttribute);

//Y Value
  const initialYAttribute = 'Retentions';
  const [yAttribute, setYAttribute] = useState(initialYAttribute);
  const yValue = d => d[yAttribute];
  const yAxisLabel = getLabel(yAttribute);

//Radius Value
  const initialradiusAttribute = 'Retentions Rate';
  const [radiusAttribute, setradiusAttribute] = useState(initialradiusAttribute);
  const radiusValue = d => d[radiusAttribute];
  
//Color Value
  const colorValue = d => d.Year;
	const colorLegendLabel = 'Year';

//Why is this needed?
  const circleRadius = 7;
  
//Checks to see if data is loaded (null)
  if (!data) {
    return <pre>Loading...</pre>;
  }
  
  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;

  const siFormat = format('.2s');
  const xAxisTickFormat = tickValue => siFormat(tickValue).replace('G', 'B');
  const yAxisTickFormat = tickValue => siFormat(tickValue).replace('G', 'B');
  
	const filteredData = data.filter(d => hoveredValue === colorValue(d));
  
  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0 ]);

  const radiusScale = scaleLinear()
    .domain(extent(data, radiusValue))
    .range([2, 15]); //range of circle sizes
  
    const colorScale = scaleOrdinal()
    .domain(data.map(colorValue))
    .range(['#183A2B', '#235740', '#2F7455', '#3B916A', '#47AE80','#60BE94','#7DCAA7']);
  
  return (
    <>
      <div className="menus-container">
        <span className="dropdown-label">X</span>
        <ReactDropdown
          options={attributes}
          value={xAttribute}
          onChange={({ value }) => setXAttribute(value)}
        />
         <span className="dropdown-label">Y</span>
        <ReactDropdown
          options={attributes}
          value={yAttribute}
          onChange={({ value }) => setYAttribute(value)}
        />
        <span className="dropdown-label">Size</span>
        <ReactDropdown
          options={attributes}
          value={radiusAttribute}
          onChange={({ value }) => setradiusAttribute(value)}
        />
      </div>
      <svg width={width} height={height}>
        <g transform={`translate(${margin.left},${margin.top})`}>
          <AxisBottom
            xScale={xScale}
            innerHeight={innerHeight}
            tickFormat={xAxisTickFormat}
            tickOffset={5}
          />

        <text
            className="axis-label"
            textAnchor="middle"
            transform={`translate(${-yAxisLabelOffset},${innerHeight /
              2}) rotate(-90)`}
          >
            {yAxisLabel}
          </text>
        
      <AxisLeft yScale={yScale} innerWidth={innerWidth} tickOffset={5} />
          <text
            className="axis-label"
            x={innerWidth / 2}
            y={innerHeight + xAxisLabelOffset}
            textAnchor="middle"
          >
            {xAxisLabel}
          </text>
        
        
        <g transform={`translate(${innerWidth + 60}, 60)`}>
            <text x={35} y={-25} className="axis-label" textAnchor="middle">
              {colorLegendLabel}
            </text>
            <ColorLegend
              tickSpacing={22}
              tickSize={10}
              tickTextOffset={12}
              tickSize={circleRadius}
              colorScale={colorScale}
              onHover={setHoveredValue}
              hoveredValue={hoveredValue}
              fadeOpacity={fadeOpacity}
            />
          </g>
 		
 		<g opacity={hoveredValue ? fadeOpacity : 1}>
          <Marks
            data={data}
            xScale={xScale}
            xValue={xValue}
            yScale={yScale}
            yValue={yValue}
            colorScale={colorScale}
            colorValue={colorValue}
            circleScale={radiusScale}
          	circleValue={radiusValue}
            tooltipFormat={xAxisTickFormat}
            circleRadius={circleRadius}
          />
        </g>
         <Marks
            data={filteredData}
            xScale={xScale}
            xValue={xValue}
            yScale={yScale}
            yValue={yValue}
            colorScale={colorScale}
          	colorValue={colorValue}
            circleScale={radiusScale}
          	circleValue={radiusValue}
            tooltipFormat={xAxisTickFormat}
            circleRadius={circleRadius}
          />
      </g>
      </svg>
    </>
  );
};
const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
